module Vector exposing (..)

import Tuple exposing (pair, first, second)
import Util exposing (maybeIf)

type alias Vector = (Float, Float)

type alias LineSegment = (Vector, Vector)

add (x1,y1) (x2,y2) = (x1+x2, y1+y2)
sub (x1,y1) (x2,y2) = (x1-x2, y1-y2)
len (x,y) = sqrt (x*x + y*y)
smul s (x,y) = (s*x, s*y)
distance v1 v2 = sub v1 v2 |> len

midpoint (x1,y1) (x2,y2) = ((x1+x2)/2, (y1+y2)/2)

normalise (x,y) =
    let
        d = len (x,y)
    in
        (x/d, y/d)

normal (x,y) = normalise (-y,x)

dot (x1,y1) (x2,y2) = x1*x2 + y1*y2

sum : List Vector -> Vector
sum = List.foldl add (0,0)

point_line_distance p (p1,p2) =
    let
        v1 = sub p2 p1
        v2 = sub p p1
        n = normal v1
        d1 = len v1
        alpha = (dot v1 v2) / d1
        d = len (sub p (add p1 (smul alpha v1)))
    in
        if alpha<0 then len (sub p p1) else if alpha>d1 then len (sub p p2) else d

closest : (a -> Vector) -> Vector -> List a -> Maybe (Int, Float, a)
closest get_position p1 = List.map (\a -> (a, get_position a |> sub p1 |> len)) >> List.indexedMap (\i (a,d) -> (i,d,a)) >> List.sortBy (\(i,d,a) -> d) >> List.head

closest_within : (a -> Float) -> (a -> Vector) -> Vector -> List a -> Maybe (Int, Float, a)
closest_within get_limit get_position p1 things =
     closest get_position p1 things 
  |> maybeIf (\(i,d,a) -> d < get_limit a)

{- The closest point to the given segment. Returns how far along the segment the point is (0 to 1), and the point -}
closest_point_on_segment : LineSegment -> Vector -> (Float,Vector)
closest_point_on_segment (p1,p2) p =
  let
    v = sub p2 p1
    vp = sub p p1
    l = len v
    d = (dot v vp) / ((len v)^2)
    t = clamp 0 1 d
  in
    (d, add p1 (smul t v))